/*****************************************************/
/*                mp3 player V2.0                                                   */
/* Description : A mp3 player support lrc sd & U disk                 */
/* Platform     : AVRStudio4.13 + WinAVR20070525  m64        */
/* Author       : Michael Zhang - 䲨                            */
/* Email         : sudazqb@163.com                                          */
/* MSN          : zhangqibo_1985@hotmail.com                          */
/* Date          : 2007-11-09                                                    */
/* NOT FOR COMMERCIAL USE,     ALL RIGHT RESERVED!         */
/*****************************************************/

/*********************  old  ***************************/
/*                   MP3/Wma/Midi                      */
/*  WinAVR 20060421                                      */
/*  ߣBozai䲨                                    */
/*  E-mailsudazqb@163.com                                  */
/*  20061212                                           */
/*************************************************************/
/*  20071109: add & modify function for any directory music file playing */
/*  20071103: add function for lyric display */


#include<avr/io.h>
#include<avr/eeprom.h>
#include"CH375/CH375.h"
#include"FAT/FAT.h"
#include"UART/UART.H"
#include"VS1003B/VS1003B.h"
#include<avr/pgmspace.h>
#include"LCD/LCD_ASCII.h"
#include"LCD/LCD.H"
#include"LCD/LCD_GBK.h"
#include"MMC_SD/MMC_SD.h"
#include"LCD/LCD_APP.h"
#include<avr/interrupt.h>
//#include"IDE/IDE.h"

#define uint unsigned int

#define ENTER_KEY 0x0d
#define BACK_KEY  0x08
#define ESC_KEY   0x1b
#define MAX       64

#define MODE _BV(PE6)
#define STOP _BV(PE7)
#define NEXT _BV(PE3)
#define UP   _BV(PE5)
#define DOWN _BV(PE4)
#define PREV _BV(PE2)


#define MP3 1
#define WMA 2
#define MID 3
#define WAV 4

//mode
#define REPET_ALL 0
#define REPET_ONE 1
#define RANDOM    2

/* enum for ICON */
enum IconTag
{
	Playing,Pause,RepetAll,RepetOne,Shuffle,Time,Music,Speaker
};

/* Long name buffer and flag */
extern unsigned char LongNameBuffer[MAX_LONG_NAME_SIZE];
extern unsigned char LongNameFlag;

/* indicate if the file system is FAT32 , ortherwise it is FAT16, note FAT12 is not supported */
extern BYTE FAT32_Enable;

extern WORD SectorsPerClust;//ÿ
extern WORD FirstDataSector;//һ

//struct FileInfoStruct FileInfo;//ļϢ

struct direntry MusicInfo;	/* Music file information, 32Bytes short directory record, contain the short name and others */
struct direntry LrcInfo;	/* lyric file information, 32Bytes short directory record, the short name and others */


/* The path of the music file & lyric file , caution: each driectory should long longer than 8, that is not support long name*/
#if FIX_DIRECTORY
#define MUSIC_PATH "\\music"
#endif

#define LRC_PATH "\\lrc"

void (*BootLoaderEntry)(void) = 0xf800;	/* boot loader entry, bootloader size: 2048Bytes */

uint16 totalsongs;	/* total number of songs*/
uint8 type;			/* current song's file type, mp3 wma mid or wav */
uint8 lrc =0;			/* Gloable variable to indicate wether the songs has a lyric file, 1 means have */
uint8 HanziEnable = 0;	/* Gloable variable to indicate wether the firmware are exist, and 1 means exist */

/* Time tag structure */
struct LrcStruct_s {
	struct LrcStruct_s * next;	/* the next node */
	uint32 time;				/* Time */
	uint16 eeaddr;			/* Address, start with 0, value greater than MAXLRCDATSIZE is the eeprom address*/
};


struct LrcStructHead_s {
	struct LrcStruct_s *header;	/* Pointer to the first node of time tag struct */
#define TI_LEN 32
	uint8 title[TI_LEN];			/* Title */
#define AR_LEN 16
	uint8 artist[AR_LEN];			/* Artist */
	uint16 offset;					/* Offset */
       uint8 sign;					/* "1" means "+", and "0" means "-" */
};

#define DEBUG 0	/* Macro for DEBUG, if 1 DEBUG message will show throw the UART */


struct LrcStructHead_s LrcStructHead;	/* Gloable struct variable to record the lyric info */

/* Use to record the time tag info */
#define MAXITEM 80
uint8 lrcbuffer[sizeof(struct LrcStruct_s) * MAXITEM];

/* Use to record the lyric data */
#define MAXLRCDATSIZE 650			/* Max data size in SRAM, other will be store in EEPROM */
uint8 lrcdatbuf[MAXLRCDATSIZE];


uint8 track[128];			/* stroe the information of songs (bit set indicate songs has been played) */

void ClearTrackInfo()		/* cleare the array track[128] */
{
	uint8 i;
	for(i=0;i<128;i++)track[i] = 0;
}

uint8 SetTrack(uint16 songs)/* set the track bit, return 1 means the song has been played */
{
	uint8 byte_offset;
	uint8 bit_offset;
	songs--;
	byte_offset = songs/8;
	bit_offset = songs%8;
	if(track[byte_offset] & (1<<bit_offset))return 1;
	else
	{
		track[byte_offset] |= 1<<bit_offset;
		return 0;
	}
}

/*void nextline()
{
	USART_putchar(0x0d);
	USART_putchar(0x0a);
}*/

void Delay(uint16 n)//ʱ
{
	while(n--)asm("nop");
}

//Timer initialization offer seed of the srandom()
void Timer1_Initial()
{
	TCNT1H=0x00;
	TCNT1L=0x00;
	TCCR1A=0x03;
	TCCR1B=0x01;
	ICR1H=0xff;
	ICR1L=0xff;
}

uint8 strcomp(uint8 * src, uint8 * dst) /* String compare */
{
	while(*src)
	{
		if(*src++ != *dst++)return 0;
	}
	return 1;
}

uint8 strcomp_noncase(uint8 * src, uint8 * dst)/*we should make sure the src is upcase*/
{
	uint8 * p1 = src, * p2 = dst;
	while(*p1)
	{
		if((*p1 == *p2) || ((*p1<*p2)&&((*p2-*p1) == 0x20 )) )
		{
			p1 ++;
			p2 ++;
		}
		else return 0;
	}
	return 1;
}

/* Lyric proccess fuction */
#if FIX_DIRECTORY
uint8 LrcProc(uint8 *LongNameBuffer) /* Parameter is the song's long name or short name, note: do not contain the extention*/
#else
uint8 LrcProc(uint8 *LongNameBuffer,WORD music_record_addr) /* Parameter is the song's long name or short name, note: do not contain the extention*/
#endif
{
	/* lyric time tag struct pointer for process*/
	struct LrcStruct_s * LrcStruct_p;
	struct LrcStruct_s * LrcStruct_p_up, * LrcStruct_p_temp,* LrcStruct_p_down;
	struct LrcStruct_s * LrcStruct_p_header;
		
	uint16 lrcaddr;		/* The address to store each lyric */
	uint8 part;			/* the sector number of one cluster */
	uint16 i;				/* loop variable for gerneral perpus */
	uint16 j;
	uint8 temp;			/* temp variable */
	uint8 * buffer;		/* buffer pointer, later will use malloc to get the data area for it*/
	uint16 min,sec,ms;	/* temp variable for time tag process, minter, second, and milisecond */
	uint8 sector;			/* sector number to record current sector, with variable j to determine wether the file is ended*/
	uint8 totalsect;		/* total sectors the file contains */
	uint16 leftbytes;		/* how many data contained in the last sector */
	uint32 p;			/* cluster number of file, next ,etc */
	uint8 mscnt;			/* counter for ms, you konw 2.3 means 2300ms not 2003ms, so need this variable to determin this*/
       
	LrcStructHead.header = (struct LrcStruct_s *)lrcbuffer;	/* set lrc info struct 's header to the buffer */

#if FIX_DIRECTORY
	if(SearchLrc(LRC_PATH,LongNameBuffer,&LrcInfo)==0)	/* search the path specified, and if found lyric file it will return 0*/
#else
	if(SearchLrc(LRC_PATH,LongNameBuffer,&LrcInfo,music_record_addr)==0)	/* search the path specified, and if found lyric file it will return 0*/
#endif
	{ 
		printf_P(PSTR("\r\n\r\n*****************************************************"));
		printf_P(PSTR("\r\nFound LRC file!"));
		printf_P(PSTR("\r\n\r\nAnalyzing LRC file ...........\r\n"));
	   
		lrc = 1;								/* Set the flag */
		lrcaddr = 0;							/* Clear the address */
	   
		LrcStruct_p = LrcStructHead.header;		/* intialize the sigle direction node */
		for(i=0;i<MAXITEM-1;i++)
		{
			LrcStruct_p->next = LrcStruct_p + 1;
			LrcStruct_p = LrcStruct_p->next;
		}
		LrcStruct_p->next = 0;
		LrcStruct_p = LrcStructHead.header;
								  
		LrcStructHead.offset = 0;				/* clear the lyc info struct */
		LrcStructHead.title[0] = '\0';
		LrcStructHead.artist[0] = '\0';
	   
		buffer = malloc(520);					/* allocate memory for read the lyric file */
		if(buffer==0)
		{
			#if DEBUG
			printf_P(PSTR("\r\nERROR: No enough memory!"));
			#endif
			return 1;
		}

		p = LrcInfo.deStartCluster+(((unsigned long)LrcInfo.deHighClust)<<16);	/* Calculate the first cluster of the lyric file */
		totalsect = LrcInfo.deFileSize/512; /*			//calculate the total sectors */
		leftbytes = LrcInfo.deFileSize%512; /*ʣֽ	//calculate the left bytes */	
		sector=0;				/* clear the sector counter */

/*  here to start analyze the lyric file */    
		while(1)
		{
			for(part=0;part<SectorsPerClust;part++)	/* first loop for read a sector of a culster */
			{
				if(FAT_LoadPartCluster(p,part,buffer))
				{
					#if DEBUG
						printf_P(PSTR("\r\nERROR: Failed to read one sector"));
					#endif
					free(buffer);
					return 1;
				}
				for(j=0;j<512;)	/* second loop for analyze each byte of the sector */
				{
					if(sector == totalsect && j == leftbytes)	/* see if it is the end of file */
					{
						part = SectorsPerClust;		/* next loop it will get out */
						break;
					}
					if(buffer[j] == '[')			/* if it is the "[" : the tag start */
					{
						#if DEBUG
							printf_P(PSTR("\r\nfound tag start ["));
						#endif
/**********************************************************************************************************/                                    
						i = 0;
						if(strcomp_noncase("TI:",&buffer[j+1]))		/* string compare to check if this is a title tag */
						{
							j+=4;		/* count add 4 */
							while(1)		/* store the title */
							{
								temp = buffer[j++];
								if(temp == 0x0d || temp == 0x0a || temp == ']' || i> (TI_LEN - 2)) /* title ended */
									break;
								LrcStructHead.title[i++] = temp;
							}
							LrcStructHead.title[i] = 0;	/* need a 0 to end the string */
                        
							#if DEBUG
								printf_P(PSTR(" found title tag: %s ]"),LrcStructHead.title);
							#else
								printf_P(PSTR("\r\nTitle  : %s "),LrcStructHead.title);
							#endif
                                            
						}
/**********************************************************************************************************/						
						else if(strcomp_noncase("AR:",&buffer[j+1]))	/* string compare to check if this is a artist tag */
						{
							j+=4;	/* count += 4 */
							while(1)	/* store the content */
							{
								temp = buffer[j++];
								if(temp == 0x0d || temp == 0x0a || temp == ']' || i>(AR_LEN - 2))	/* tag ended */
									break;
								LrcStructHead.artist[i++] = temp;
							}
							LrcStructHead.artist[i] = 0;		/* need a 0 to end the string */
                        
							#if DEBUG
   								printf_P(PSTR(" found artist tag: %s ]"),LrcStructHead.artist);
							#else
								printf_P(PSTR("\r\nArtist : %s "),LrcStructHead.artist);
							#endif
	 
						}
/**********************************************************************************************************/
						else if(strcomp_noncase("OFFSET:",&buffer[j+1]))	/* string compare to check if this is a offset tag */
						{
							j+=8;	/* count += 8 */
						
							LrcStructHead.offset = 0;	/* default value, in case the tag like this [offset:] */
 							LrcStructHead.sign = 1;

							#if DEBUG
								printf_P(PSTR(" found offset tag: "));
							#else
								//printf_P(PSTR("\r\n"));
							#endif
                                           
							while(1)	/* proceed the offset */
							{
								temp = buffer[j++];

								#if DEBUG
									USART_putchar(temp);
								#endif
                                                       
								if(temp == ']')break;	/* tag end */
								if(temp == '-')			/* sign */
								{
									LrcStructHead.sign = 0;
									continue;
								}
								LrcStructHead.offset *= 10;			/* calculate the value */
								LrcStructHead.offset += (temp-0x30);
							}
                                           
							#if DEBUG
								printf_P(PSTR("the value in the structure is : %d"),LrcStructHead.offset);
							#endif
                                             
						}
/**********************************************************************************************************/					
						else if(strcomp_noncase("BY:",&buffer[j+1]))	/* string compare to check if this is a By tag */
						{											/* but we do not need this tag */
							j+=4;
                                              
							#if DEBUG
								printf_P(PSTR(" found by tag: "));
							#endif
                                               
							while(1)
							{
								temp = buffer[j++];

								#if DEBUG
									USART_putchar(temp);
								#endif
                                                       
								if(temp == 0x0d || temp == 0x0a || temp == ']' ||i>30)break;
							}
						}
/**********************************************************************************************************/									 
						else if(strcomp_noncase("AL:",&buffer[j+1]))	/* string compare to check if this is a album tag */
						{
							j+=4;

							#if DEBUG
								printf_P(PSTR(" found al tag: "));
							#else
								printf_P(PSTR("\r\nAlbum  : "));
							#endif

							while(1)
							{
								temp = buffer[j++];   
								if(temp == 0x0d || temp == 0x0a || temp == ']' ||i>30)break;
								USART_putchar(temp);
							}
						}
/**********************************************************************************************************/									   
						else		/* otherwise this must the time tag or other not recognize tag */
						{
							j+=1;
							/****************************************************************/
							if(sector == totalsect && j == leftbytes + 1) goto end; /* if the file is end */
							if(j == 512)	/* wether the sector is end, need to read the next sector */
							{
								if(part == (SectorsPerClust - 1))	/* if need to read next cluster */
								{
									//p=FAT_NextCluster_NEW(p);//һ			//read next cluster
									p=FAT_NextCluster(p);
									if(p == 0x0fffffff || p == 0x0ffffff8 || (FAT32_Enable == 0 && p == 0xffff))//޺	//no more cluster  
									{
										#if DEBUG
											printf_P(PSTR("\r\nNo next cluster or read next cluster error"));
										#endif
										goto end;
									}
									part = 0;	/* clear the part count */
									if(FAT_LoadPartCluster(p,part,buffer))
									{
										#if DEBUG
											printf_P(PSTR("\r\nERROR: Faild to load a sector"));
										#endif
										free(buffer);
										return 1;
									}
								}
								else 	/* else just read the next part of the cluster */
								{
									part ++;	/* part ++ */
									if(FAT_LoadPartCluster(p,part,buffer))
									{
										#if DEBUG
											printf_P(PSTR("\r\nERROR: Faild to load a sector"));
										#endif
										free(buffer);
										return 1;
									}
								}
								sector ++;	/* sector count +=1 */
								j = 0;		/* loop value clear to 0*/
							}
							/********************************************************/											  
							min = 0;		/* clear the temp variable for time to 0 */
							sec = 0;
							ms = 0;
							i = 0;		/* for tag use, to ditermin tag like [XX:XX] & [XX:XX:XX] */
							while(1)
							{
								temp = buffer[j++];

								#if DEBUG
									USART_putchar(temp);
								#endif
														  
								/**********************************************************/
								if(sector == totalsect && j == leftbytes + 1) goto end;	/* file end */
								if(j == 512)		/* need to read next sector */
								{
									if(part == (SectorsPerClust - 1))	/* if need to read next cluster */
									{
										//p=FAT_NextCluster_NEW(p);//һ			//read next cluster
										p=FAT_NextCluster(p);
										if(p == 0x0fffffff || p == 0x0ffffff8 || (FAT32_Enable == 0 && p == 0xffff))//޺	//no more cluster  
										{
											#if DEBUG
												printf_P(PSTR("\r\nNo next cluster or read next cluster error"));
											#endif
											goto end;
										}
										part = 0;	/* clear the part count */
										if(FAT_LoadPartCluster(p,part,buffer))
										{
											#if DEBUG
												printf_P(PSTR("\r\nERROR: Faild to load a sector"));
											#endif
											free(buffer);
											return 1;
										}
									}
									else 	/* else just read the next part of the cluster */
									{
										part ++;	/* part ++ */
										if(FAT_LoadPartCluster(p,part,buffer))
										{
											#if DEBUG
												printf_P(PSTR("\r\nERROR: Faild to load a sector"));
											#endif
											free(buffer);
											return 1;
										}
									}
									sector ++;	/* sector count +=1 */
									j = 0;		/* loop value clear to 0*/
								}
								/***********************************************/														  
								if((temp < '0' || temp > '9') && temp != ':' && temp != '.')break;	/* end or not a leagal ch*/
								if(temp == ':'){i=1;continue;}
								if(temp == '.'){i=2;mscnt = 0;continue;}

								/* calculate the value  */
								if(i==0)min = min*10 + (temp-0x30);
								else if(i==1)sec = sec*10 + (temp-0x30);
								else 
								{
									if(mscnt == 0)
										ms += (temp-0x30)*100;
									else if(mscnt == 1)
										ms += (temp - 0x30)*10;
									else ms += temp-0x30;
									mscnt ++;
								}							 
							}
							
							if(temp == ']')	/* if tag end */
							{
							
								#if DEBUG
									printf_P(PSTR("  found time tag: "));
									printf_P(PSTR("min = %d; "),min);
									printf_P(PSTR("sec = %d; "),sec);
									printf_P(PSTR("ms = %d; \r\n"),ms);
								#endif
													  
								if(LrcStruct_p == 0)
								{
									printf_P(PSTR("\r\nWARNING: Sorry! no more space for the rest tag, som lryic can't display!"));
									goto end;
								}

								/* set time value without offset */
								LrcStruct_p->time = (uint32)(min*60 + sec)*1000 + ms;
													  
								#if DEBUG
									printf_P(PSTR("\r\noffset : %d "),LrcStructHead.offset);
									printf_P(PSTR("  total  = %ld; "),LrcStruct_p->time);
								#endif

								/* recalculate the time, use offset */
								if(LrcStructHead.sign)
								{
									if(LrcStruct_p->time<LrcStructHead.offset)LrcStruct_p->time = 0;
									else LrcStruct_p->time -= LrcStructHead.offset;
								}
								else LrcStruct_p->time += LrcStructHead.offset;
													  
								#if DEBUG
									printf_P(PSTR("\r\nafter calculate total  = %ld; "),LrcStruct_p->time);
								#endif

								/* node operation, pointer to next node, and set address */
								LrcStruct_p->eeaddr = lrcaddr;
								temp = buffer[j];
								LrcStruct_p = LrcStruct_p->next;
                                                      
                                                      
								if(temp == '[');	/* if two tag share one lyric, just jump and next loop it will process it */
								else			/* this is a sigle time tag */
								{
									while(1)
									{
										temp = buffer[j++];
                                                                    
										#if DEBUG
											USART_putchar(temp);
										#endif

										/*************************************************************/
										if(sector == totalsect && j == leftbytes + 1) goto end;	/* file end */
                                                                    
										if(j == 512)	/* need to read next sector */
										{
											if(part == (SectorsPerClust - 1))	/* if need to read next cluster */
											{
												//p=FAT_NextCluster_NEW(p);//һ			//read next cluster
												p=FAT_NextCluster(p);
												if(p == 0x0fffffff || p == 0x0ffffff8 || (FAT32_Enable == 0 && p == 0xffff))//޺	//no more cluster  
												{
													#if DEBUG
														printf_P(PSTR("\r\nNo next cluster or read next cluster error"));
													#endif
													goto end;
												}
												part = 0;	/* clear the part count */
												if(FAT_LoadPartCluster(p,part,buffer))
												{
													#if DEBUG
														printf_P(PSTR("\r\nERROR: Faild to load a sector"));
													#endif
													free(buffer);
													return 1;
												}
											}
											else 	/* else just read the next part of the cluster */
											{
												part ++;	/* part ++ */
												if(FAT_LoadPartCluster(p,part,buffer))
												{
													#if DEBUG
														printf_P(PSTR("\r\nERROR: Faild to load a sector"));
													#endif
													free(buffer);
													return 1;
												}
											}
											sector ++;	/* sector count +=1 */
											j = 0;		/* loop value clear to 0*/
										}
										/****************************************************************/
										 
										if(temp == 0x0d || temp == 0x0a)	/* one lyric end, when meet next line*/
										{
											if(lrcaddr<MAXLRCDATSIZE)	/* if lrcdatbuf is not full */
											{
												lrcdatbuf[lrcaddr++] ='\0';
											}
											else						/* otherwise use the eeprom */
												eeprom_write_byte(lrcaddr++ - MAXLRCDATSIZE,'\0');
											break;
										}
										else 				/* else normal data */
										{
											if(lrcaddr == MAXLRCDATSIZE)	/* if ram buffer run out, show some messages */
											{
												printf_P(PSTR("\r\n\r\nInternal RAM is not enough, some data will be stored at EEPROM!"));
												printf_P(PSTR("\r\nThis may take little longer time. U know 8.4ms/Byte, pls wait!......"));
											}
											#if FIX_DIRECTORY
											if(lrcaddr == MAXLRCDATSIZE + 2048)
											#else
											if(lrcaddr == MAXLRCDATSIZE + RECORD_ADDR_START)
											#endif
											{
												printf_P(PSTR("\r\n\r\nWow! The lyric is so long, EEPROM is run out, sorry!\r\n"));
												goto end;
											}
											else
											{
												if(lrcaddr<MAXLRCDATSIZE)		/* use ram */
												{
													lrcdatbuf[lrcaddr++] = temp;
												}
												else							/* use epprom */
													eeprom_write_byte(lrcaddr++ - MAXLRCDATSIZE,temp);
											}
										}
									}
								} 
							}
						}
					}
					else j++;	/**/
				}
				sector ++;	/**/
			}
			p=FAT_NextCluster(p);
			//p=FAT_NextCluster_NEW(p);//һ			//read next cluster
			if(p == 0x0fffffff || p == 0x0ffffff8 || (FAT32_Enable == 0 && p == 0xffff))//޺	//no more cluster  
			{
				#if DEBUG
					printf_P(PSTR("\r\nNo next cluster or read next cluster error"));
				#endif
				break;
			}
		}
	
end:
		/* in case that some lyc file don not have empty line at the last */
		if(lrcaddr<MAXLRCDATSIZE)
		{
			lrcdatbuf[lrcaddr++] = '\0';
		}
		else
			eeprom_write_byte(lrcaddr++ - MAXLRCDATSIZE,'\0');

		/* end the node  */
		LrcStruct_p = LrcStruct_p - 1;
		LrcStruct_p->next = 0;

		/* free the buffer  */
		free(buffer);

		/* below is a debug message */
		/************************************************************/
		#if DEBUG
			printf_P(PSTR("\r\nLyric data stored completed!"));

			printf_P(PSTR("\r\nThe data stored in RAM & EEPROM:\r\n"));
			for(j=0;j<lrcaddr;j++)
			{
				if(j<MAXLRCDATSIZE)USART_putchar(lrcdatbuf[j]);
				else USART_putchar(eeprom_read_byte(j - MAXLRCDATSIZE));
			}
                        
			printf_P(PSTR("\r\nNo sorted data is : \r\n"));
			LrcStruct_p = LrcStructHead.header;
			while(1)
			{
				lrcaddr = LrcStruct_p->eeaddr;
				while(1)
				{
					if(lrcaddr<MAXLRCDATSIZE)temp = lrcdatbuf[lrcaddr++];
					else temp = eeprom_read_byte(lrcaddr++ - MAXLRCDATSIZE);
					if(temp == 0)break;
					USART_putchar(temp);
				}
				printf_P(PSTR("\r\n"));
				if(LrcStruct_p->next == 0)break;
				LrcStruct_p = LrcStruct_p->next;
			}
		#endif

		#if DEBUG
			printf_P(PSTR("\r\nSort begining ......"));
		#endif
                     
		#if DEBUG
			temp = 0;
			LrcStruct_p = LrcStructHead.header;
			while(1)
			{
				temp++;
				if(LrcStruct_p->next == 0)break;
				LrcStruct_p = LrcStruct_p->next;
			}
			printf_P(PSTR("\r\ntotal %s time tags"),temp);
		#endif
		/***************************************************************/

		/* tag sort */		   
		LrcStruct_p_header = LrcStruct_p = LrcStructHead.header;
		while(1)
		{
			LrcStruct_p_header = LrcStruct_p;
			LrcStruct_p = LrcStruct_p->next;
			if(LrcStruct_p == 0)break;
			if(LrcStruct_p_header->time > LrcStruct_p->time)
			{
				LrcStruct_p_temp = LrcStructHead.header;
				j = 0;
				while(1)
				{   
					j++;
					if(LrcStruct_p_temp->time < LrcStruct_p->time)	/* do not need to insert node */
					{
						LrcStruct_p_up = LrcStruct_p_temp;
						LrcStruct_p_temp = LrcStruct_p_temp->next;
						LrcStruct_p_down = LrcStruct_p_temp->next;
					}
					else										/* need insert */
					{
                                                   
						#if DEBUG
							printf_P(PSTR("\r\nLrcStruct_p_up->time = %ld"),LrcStruct_p_up->time);
							printf_P(PSTR("\r\nLrcStruct_p_temp->time = %ld"),LrcStruct_p_temp->time);
							printf_P(PSTR("\r\nLrcStruct_p_down->time = %ld"),LrcStruct_p_down->time);
							printf_P(PSTR("\r\nLrcStruct_p->time = %ld \r\n"),LrcStruct_p->time);
						#endif
                                                    
						if(j == 1)	/* if the first need to replace */
						{
							LrcStructHead.header = LrcStruct_p;
							LrcStruct_p_header->next = LrcStruct_p ->next;
							LrcStruct_p ->next = LrcStruct_p_header;
							LrcStruct_p = LrcStruct_p_header;
						}
						else		/* other normal condition */
						{
							LrcStruct_p_header->next = LrcStruct_p->next;
							LrcStruct_p_up->next = LrcStruct_p;
							LrcStruct_p->next = LrcStruct_p_temp;
							LrcStruct_p = LrcStruct_p_header;
                                                             
							#if DEBUG
								printf_P(PSTR("\r\nLrcStruct_p->time = %ld \r\n\r\n"),LrcStruct_p->time);
							#endif
						}
						break;
					}
				}
			}
		}
		printf_P(PSTR("\r\n\r\nAnalyze complete!\r\n\r\nCode written by Bozai - 䲨"));
		printf_P(PSTR("\r\n*****************************************************\r\n"));

		/* debug message, show sorted data */
		#if DEBUG
			printf_P(PSTR("\r\nSort completed!"));
			printf_P(PSTR("\r\nData after sort is : \r\n"));
			printf_P(PSTR("\r\n\r\nLrc Data:\r\n\r\n"));
			LrcStruct_p = LrcStructHead.header;
			while(1)
			{
				lrcaddr = LrcStruct_p->eeaddr;
				while(1)
				{
					if(lrcaddr<MAXLRCDATSIZE)temp = lrcdatbuf[lrcaddr++];
					else temp = eeprom_read_byte(lrcaddr++ - MAXLRCDATSIZE);
					if(temp == 0)break;
					USART_putchar(temp);
				}
				printf_P(PSTR("\r\n"));
				if(LrcStruct_p->next == 0)break;
				LrcStruct_p = LrcStruct_p->next;
			}
		#endif

	}
	else lrc = 0;
}

uint8 PlayMusicwithKey()//ֺһִв˳
{
	uint16 keylen;			//ڼ		//for key processing
	uint16 count;			//ݼ			//data counting
	uint8 i;				//ѭ			//loop variable
	uint16 j;				//ѭ			//loop variable
	DWORD p;				//ָʾֵ			//cluster
	DWORD totalsect;		//ļӵе	//cotain the total sector number of a file
	uint16 leftbytes;		//ʣֽ			//cotain the left bytes number of a file //the last cluster usually not fully occupied by the file
	uint8 *buffer;			//				//buffer
	DWORD sector;			//				//recor the current sector to judge the end sector
	uint8 flag;				///ͣ־		//flag of pause
	uint16 vol= DEFAULT_VOLUME;//ʼӦvs1003ĳʼͬ	//default volume
	uint8 vol_temp;
	uint16 songs=1;			//ĬϷŵһ׸	//play the fist songs by default
	uint16 songs_cnt = 0;					//how many songs have been played
	uint8 mode=0;			//ظ			//repet all by default
	unsigned long rand_val;
	
	uint8 total[6];	/* for total songs indication */
	uint8 current[12];	/* Current songs number */
	uint16 temp = totalsongs;

	uint8 time[6];
	uint16 decodetime;

	uint8 volume[3];
	uint8 min,sec;
        
	struct LrcStruct_s * LrcStruct_pp;
	uint16 lrcaddr;

	/* the display buffer (characters) */
	#define MAX_LRC_DISP 52
	uint8 lrcdata[MAX_LRC_DISP];

	ClearTrackInfo();

      
	printf_P(PSTR("\r\nCurrent mode : "));
	switch(mode)
	{
		case REPET_ALL: printf_P(PSTR("repet all"));break;
		case REPET_ONE:printf_P(PSTR("repet one"));break;
		case RANDOM:printf_P(PSTR("shuffle"));break;
	}

	if(totalsongs==0)//ûи쳣˳	//if no music file return
	{
		lcdClrDisBuf();
		LCD_print12_P(0,20,PSTR("File not found!"));
		lcdUpdateDisplay();
		return;
	}
	
/**********************************************/
	for(i=0;i<5;i++)
	{
		total[4-i] = temp%10;
		temp /= 10;
	}
	for(i=0;i<5;i++)
	{
		if(total[i])break;
	}
	for(j=0;j<5-i;j++)
	{
		total[j] = total[j+i] + 0x30;
	}
	total[j] = 0;
/**********************************************/
	vol_temp = 100-((DEFAULT_VOLUME & 0x00ff)*100/255);
	volume[0] = vol_temp/10 + 0x30;
	volume[1] = vol_temp%10 +0x30;    
	volume[2] = '\0'; 
/**********************************************/

	Timer1_Initial();//ʱڲ	//initialize the timer
next://һ׸ʼط						//label for "goto"
	if(mode==RANDOM)//Ÿ 				//if the mode is shuffle the songs
	{
		songs_cnt++;
		if(songs_cnt == totalsongs && totalsongs<1025)
		{
			ClearTrackInfo();
			songs_cnt = 0;
		}
		rand_val = TCNT1;
		Delay((random() && 0x00ff));
		rand_val <<= 16;
		rand_val += TCNT1;
		srandom(rand_val);
		if(totalsongs>1024)
		{
			songs = (uint16)(((random()/214749)*(uint32)totalsongs)/10000)+1;//	//create random song number
		}
		while(totalsongs<1025)
		{
			songs = (uint16)(((random()/214749)*(uint32)totalsongs)/10000)+1;//	//create random song number
			if(SetTrack(songs) == 0)break;
		}	
	}
	count=0;//	//clear count
	flag=1;
	if(type != MID)VS1003B_Fill2048Zero();

	VS1003B_SoftReset();//soft reset //in case of playing wma files//λ

#if FIX_DIRECTORY
	Search((unsigned char *)(MUSIC_PATH),&MusicInfo,&songs,&type);		//ҵӦļ		//find the file
#else
	Search(&lrcaddr,&MusicInfo,&songs,&type);		//ҵӦļ		//find the file
#endif

	/* here we use lrcaddr to get the record address in the eeprom which the file placed */

	for(j=0;j<MAX_LONG_NAME_SIZE/2;j++)			/* cut off the extension, only leave the name*/
	{
		if(((unsigned int*)LongNameBuffer)[j] == 0)
		{
			((unsigned int*)LongNameBuffer)[j-4] = 0;
			break;
		}
	}
/****************************************************/
	time[0] = '0';
	time[1] = '0';
	time[2] = ':';
	time[3] = '0';
	time[4] = '0';
	time[5] = '\0';
	decodetime = 0;
/******************************************************/
	temp = songs;
	for(i=0;i<5;i++)
	{
		current[4-i] = temp%10;
		temp /= 10;
	}
	for(i=0;i<5;i++)
	{
		if(current[i])break;
	}
	for(j=0;j<5-i;j++)
	{
		current[j] = current[j+i] + 0x30;
	}
	current[j++] = '/';
	for(i=0;i<6;i++)
	{
		current[j+i] = total[i];
	}
	
/********************************************************/	
	lcdClrDisBuf();
	switch(type)
	{
		case MP3:LCD_print12_P(0,0,PSTR("mp3"));break;
		case WMA:LCD_print12_P(0,0,PSTR("wma"));break;
		case MID:LCD_print12_P(0,0,PSTR("mid"));break;
		case WAV:LCD_print12_P(0,0,PSTR("wav"));break;
		default:break;
	}
	Print_ICON_12(18,0,Playing);
	LCD_print12(30,0,current);
	switch(mode)
	{
		case REPET_ALL:Print_ICON_12(72,0,RepetAll);break;
		case REPET_ONE:Print_ICON_12(72,0,RepetOne);break;
		case RANDOM   :Print_ICON_12(72,0,Shuffle);break;
		default:break;
	}
/*****************************************************/
	//LCD_print12(30,36,time);
	//LCD_print12(72,36,volume);
	//Print_ICON_12(60,36,Speaker);
	//Print_ICON_12(0,12,Music);	
/******************************************************/
	if(LongNameFlag)
	{
		if(HanziEnable)	/* if HanziEnable, convert the long name from unicode to GBK */
		{
			buffer = malloc(MAX_LONG_NAME_SIZE);
			if(buffer == 0)return 1;
			i = 0;
			j=0;
			while(1)
			{
				temp = ((unsigned int *)LongNameBuffer)[i++];

				/*  you may confused about following code, because at previous we set "." to 0 for lry search*/
				if(temp == 0 )
				{
					buffer[j++] = '.';
					while(1)	/*copy the extention */
					{
						temp = ((unsigned int *)LongNameBuffer)[i++];
						if(temp == 0)break;	/* end */
						buffer[j++] = (uint8)temp;
					}
					break;
				}
				else if (temp<0x80)
				{
					buffer[j++]=(uint8)temp;
				}
				else if(temp<0x4e00)
				{
					*((unsigned int*)&(buffer[j])) = "?";
					j+=2;
				}
				else if(temp<0x9fa6)
				{
					if(Unicode_to_GBK((unsigned char *)&temp))return 1;
					*((unsigned int*)&(buffer[j])) = temp;
					j+=2;                                
				}
				else
				{
					*((unsigned int*)&(buffer[j])) = "?";
					j+=2;
				}
			}
			buffer[j] = 0;
			LCD_print12(12,12,buffer);
			//LCD_print_unicode12(12,12,(unsigned int *)LongNameBuffer);
		}
		else
		{
			LCD_print12_P(0,12,PSTR("-- --By bozai\nFirmware lost!"));
		}
	}
	else 	/* short name */
	{
		for(j=0;j<8;j++)
		{
			if(MusicInfo.deName[j]==0x20)break;
			LongNameBuffer[j]=MusicInfo.deName[j];
		}
		//LongNameBuffer[j++]='\0';
		LongNameBuffer[j++]='.';
		count = j+3;
		for(count=0;count<3;count++)LongNameBuffer[j+count]=MusicInfo.deExtension[count];
		LongNameBuffer[j+count]='\0';
		if(HanziEnable)LCD_print12(12,12,LongNameBuffer);
		else
		{
			LCD_print12_P(0,12,PSTR("    -By bozai\nFirmware lost!"));
		}
	} 
/*************************************************************/
	LCD_print12(30,36,time);
	LCD_print12(72,36,volume);
	Print_ICON_12(60,36,Speaker);
	Print_ICON_12(0,12,Music);	
	lcdUpdateDisplay();
/**************************************************************/

	printf_P(PSTR("\r\nNow playing "));
	printf_P(PSTR("%s :"),current);
	if(LongNameFlag && HanziEnable)
	{
		printf_P(PSTR("%s"),buffer);
		free(buffer);
	}
	else
	{
		for(j=0;j<8;j++)
		{
			if(MusicInfo.deName[j]==0x20)break;
			USART_putchar(MusicInfo.deName[j]);
		}
        
		USART_putchar('.');
		for(j=0;j<3;j++)
		{
			//if(MusicInfo.deName[j]==0x20)break;
			USART_putchar(MusicInfo.deExtension[j]);
		}
	}

/**************************************************************/
#if FIX_DIRECTORY
	LrcProc(LongNameBuffer);
#else
	LrcProc(LongNameBuffer,lrcaddr);			/* search lyc file */
											/* lrcaddr indicate the record address of eeprom */
											/* first search the current folder then the default */
#endif											
	LrcStruct_pp = LrcStructHead.header;

/**************************************************************/

	p     = MusicInfo.deStartCluster+(((unsigned long)MusicInfo.deHighClust)<<16);//ļ״	//the first cluster of the file
		
	totalsect = MusicInfo.deFileSize/512; //			//calculate the total sectors
	leftbytes = MusicInfo.deFileSize%512; //ʣֽ	//calculate the left bytes	
	i=0;
	sector=0;

	if(lrc && HanziEnable)		/* clear the previous time area */
	{
		LCD_print12_P(36,36,PSTR("    "));
		lcdClrDisBuf(); 
	}
    
	while(1)
	{
		keylen=0;
    		for(;i<SectorsPerClust;i++)		//һ	//a cluster
		{
			buffer=malloc(512);
			FAT_LoadPartCluster(p,i,buffer);//һ	//read a sector
			count=0;
			while(count<512)
			{
				if(flag==0){if(keylen){Delay(100);keylen--;}}
				else if(type == MID){if(keylen){Delay(100);keylen--;}}

				if(VS1003B_NeedData() && flag)	//Ҫ  //send data  honoring DREQ
				{
					VS1003B_Write32B(&buffer[count]);
					count += 32;
					if(keylen)keylen--;			//ڼ			//for key processing
					if(sector == totalsect && count >= leftbytes)		//ļѽ		//if this is the end of the file
					{
						if(type == MID)//waiting the midi file was decoded
							VS1003B_Fill2048Zero();
						i=SectorsPerClust;
						break;
					}//ļ			//file ended
					if(count == 511){break;}//512ֽ		//break if a sector was sent
				}
				else if(decodetime == 0 || VS1003B_ReadDecodeTime() != decodetime)
				{
					decodetime = VS1003B_ReadDecodeTime();
					min = decodetime/60;
					sec = decodetime%60;
                                    if(lrc)
                                    { 
                                             if(min>99);
				        	   else if(HanziEnable)
				        	  {
				        		time[0] = min/10 + 0x30;
				        		time[1] = min%10 + 0x30;
				        		time[3] = sec/10 + 0x30;
				        		time[4] = sec%10 + 0x30;
				        		LCD_print12_fast(54,36,time);
				        	  } 
						else
						{
							time[0] = min/10 + 0x30;
				        		time[1] = min%10 + 0x30;
				        		time[3] = sec/10 + 0x30;
				        		time[4] = sec%10 + 0x30;
				        		LCD_print12_fast(30,36,time);
						}
						min = 0;
						while(1)
						{
							if(LrcStruct_pp->time/1000 <= decodetime && LrcStruct_pp != 0)
							{
								lrcaddr = LrcStruct_pp->eeaddr;
								while(1)
								{
									if(lrcaddr<MAXLRCDATSIZE)temp = lrcdatbuf[lrcaddr++];
									else temp = eeprom_read_byte(lrcaddr++ - MAXLRCDATSIZE);

									if(temp == 0)
									{
										break;
									}
									if(min<MAX_LRC_DISP-1)lrcdata[min++] = temp;
								}
								lrcdata[min] = '\n';
								LrcStruct_pp = LrcStruct_pp->next;
							}
							else
							{
								if(min && HanziEnable)
								{
									lcdClrDisBuf(); 
									lrcdata[min] = '\0';
									printf_P(PSTR("\r\n[%s] %s"),time,lrcdata);
									LCD_print12_fast(0,0,lrcdata);
									LCD_print12_fast(54,36,time);
								}
								else if(min)
								{
									lrcdata[min] = '\0';
									printf_P(PSTR("\r\n[%s] %s"),time,lrcdata);
									LCD_print12_fast(30,36,time);
								}
								break;
							}
						}
						lcdUpdateDisplay();
					}
					else
					{
						if(min>99);
						else
						{
							time[0] = min/10 + 0x30;
							time[1] = min%10 + 0x30;
							time[3] = sec/10 + 0x30;
							time[4] = sec%10 + 0x30;
							LCD_print12(30,36,time);
							lcdUpdateDisplay();
						} 
					}
				}
			#if 0
				if((PIND&STOP)==0 /*&& keylen==0*/)//ͣ		//key PLAY/PAUSE
				{
					Delay(100);
					if(!(PIND&STOP))
					{
						while(!(PIND&STOP));
						if(flag)flag=0;
						else flag=1;
						Delay(1000);
					}
				}
				else if(!(PIND&DOWN) && keylen==0)  //-			//Volume down
			#else
				if(!(PINE & DOWN) && keylen==0)  //-
			#endif
				{
					Delay(100);
	   			 	if(!(PINE & DOWN))  
					{
						keylen=200;
						if(vol_temp == 0);
						else 
						{
							vol_temp--;
							vol = (uint8)(255-(((uint16)vol_temp)*255)/100);
							vol = (vol<<8) +vol;
							VS1003B_SetVolume(vol);
							volume[0] = vol_temp/10 + 0x30;
							volume[1] = vol_temp%10 +0x30; 
							printf_P(PSTR("\r\nTurn volume down to: %s"),volume);
							if(lrc == 0 || HanziEnable == 0)
							{
								LCD_print12(72,36,volume);
								lcdUpdateDisplay();
							}
							else
							{
								Print_ICON_12(30,36,Speaker);
								LCD_print12(42,36,volume);
								lcdUpdateDisplay();
							}
						} 
					}
				}
				else if(!(PINE & UP) && keylen==0) //+			//Volume up
				{
					Delay(100);
					if(!(PINE & UP)) 
					{
						keylen=200;
						if(vol_temp == 99);
						else  
						{
							vol_temp++;
							vol = (uint8)(255-(((uint16)vol_temp)*255)/100);
							vol =(vol<<8) + vol;
							VS1003B_SetVolume(vol);
							volume[0] = vol_temp/10 + 0x30;
							volume[1] = vol_temp%10 +0x30;
							printf_P(PSTR("\r\nTurn volume up   to: %s"),volume);
							if(lrc == 0 || HanziEnable == 0)
							{
								LCD_print12(72,36,volume);
								lcdUpdateDisplay();
							}
							else
							{
								Print_ICON_12(30,36,Speaker);
								LCD_print12(42,36,volume);
								lcdUpdateDisplay();
							}
						} 
					}
				}
				else if(!(PINE & NEXT)) //һ						next songs
				{
					Delay(50000);		
					if(!(PINE & NEXT))
					{
						while(!(PINE & NEXT));
						songs++;
						if(songs > totalsongs)songs=1;
						{
							free(buffer);
							goto next;
						}
					}
				}
				else if(!(PINE & PREV)) //һ						previous songs
	 			 {
					Delay(5000);		
					if(!(PINE & PREV))
					{
						while(!(PINE&PREV));
						if(songs == 1)songs=totalsongs;
						else songs--;
						{
							free(buffer);
							goto next;
						}
					}
	  			 }
				else if((!(PINE & MODE)) && keylen==0) //ģʽ			//mode key
	 			{
					Delay(100);		
					if(!(PINE & MODE))
					{
						keylen=0xffff;
						if(mode==REPET_ALL)
						{
							mode=REPET_ONE;
						//	LED1_ON();
						//	LED2_OFF();
						}
						else if(mode==REPET_ONE)//next mode is shuffle
						{
							mode=RANDOM;
						//	LED1_OFF();
						//	LED2_ON();
							srandom(((unsigned long)TCNT1)<<16);//
						}
						else 
						{
							mode=REPET_ALL;
						//	LED1_OFF();
						//	LED2_OFF();
						}
						if(lrc == 0 || HanziEnable == 0)
						{
							switch(mode)
							{
								case REPET_ALL:Print_ICON_12(72,0,RepetAll);printf_P(PSTR("\r\nSet mode -> repet all"));break;
								case REPET_ONE:Print_ICON_12(72,0,RepetOne);printf_P(PSTR("\r\nSet mode -> repet one"));break;
								case RANDOM   :Print_ICON_12(72,0,Shuffle);printf_P(PSTR("\r\nSet mode -> shuffle"));break;
								default:break;
							}
							lcdUpdateDisplay();
						}
						else
						{
							switch(mode)
							{
								case REPET_ALL:Print_ICON_12(18,36,RepetAll);printf_P(PSTR("\r\nSet mode -> repet all"));break;
								case REPET_ONE:Print_ICON_12(18,36,RepetOne);printf_P(PSTR("\r\nSet mode -> repet one"));break;
								case RANDOM   :Print_ICON_12(18,36,Shuffle);printf_P(PSTR("\r\nSet mode -> shuffle"));break;
								default:break;
							}
							lcdUpdateDisplay();
						}  
					}
				}
			}
			sector++;
			free(buffer);
		}
		i=0;
              p=FAT_NextCluster(p);
		//p=FAT_NextCluster_NEW(p);//һ			//read next cluster
		if(p == 0x0fffffff || p == 0x0ffffff8 || (FAT32_Enable == 0 && p == 0xffff))//޺	//no more cluster
		{
			if(mode==REPET_ALL)songs++;
			if(songs>totalsongs)songs=1;
			goto next;
		}
	}
}

int main()
{
	DDRE = 0x00;
	PORTE = 0xff;
	PORTB |= 1<<PB0;	/* we should make sure that AVR's SS pin is held high no mater the spi is master or slave*/
	
	OSCCAL = 0xc5;

	Delay(0xffff);

	COM_Initial(MYUBRR);//ʼ

	printf_P(PSTR("\r\n************************************************"));
	printf_P(PSTR("\r\n          SD & USB disk mp3 player V2.0                            "));
	printf_P(PSTR("\r\n   Design by Michael Zhang (bozai) 䲨                 "));
	printf_P(PSTR("\r\n************************************************\r\n"));

	if(VS1003B_Init())
	{
		printf_P(PSTR("\r\nVS1003 config fialed!"));//VS1003
	}
	else 
	{
		printf_P(PSTR("\r\nVS1003 config OK!"));
	}
	
	printf_P(PSTR("\r\nLCD initialize OK!"));
	lcdInit();

	CH375_PORT_INI();
	if(CH375_CheckExist())
	{
		printf_P(PSTR("\r\nCH375 port initialize OK! & find the chip!"));
	}
	else printf_P(PSTR("\r\nCH375 chip not exist!"));

	lcdClrDisBuf();
	LCD_print12_P(0,0,PSTR("Please press\nNEXT for SD\nPREV for USB\n    --by bozai"));
	lcdUpdateDisplay();

	printf_P(PSTR("\r\n"));
	printf_P(PSTR("\r\nPlease press:"));
	printf_P(PSTR("\r\n             \"NEXT\" key to choose SD card."));
	printf_P(PSTR("\r\n             \"PREV\" key to choose USB Disk!"));
	printf_P(PSTR("\r\n             \"UP\" to enter bootloader     in 10 seconds!"));
	printf_P(PSTR("\r\n             Otherwise it will execute the bootloader code automaticly!\r\n"));
	
	totalsongs = 0;	/* here for loop variable */
	while(PINE & NEXT && (PINE & PREV) && (PINE & UP))
	{
		if (totalsongs++ == 10000)
		{
			goto BootLoader;
		}
		_delay_ms(1);
	}
	totalsongs = 0;	/* clear it for the following code use */

	if(!(PINE & NEXT))
	{
		lcdClrDisBuf();
		LCD_print12_P(0,0,PSTR("Pls insert SD card!"));
		lcdUpdateDisplay();
		printf_P(PSTR("\r\nPlease insert SD card!"));
		while(MMC_SD_Init());
		printf_P(PSTR("\r\nSD initialize OK!"));
		FAT_ReadSector = MMC_SD_ReadSingleBlock;//device read
		FAT_WriteSector = MMC_SD_WriteSingleBlock;//device write
		FAT_ReadCapacity = MMC_SD_ReadCapacity;//read capacity
	}
	else if(!(PINE & PREV))
	{
		lcdClrDisBuf();
		LCD_print12_P(0,0,PSTR("Pls insert U disk!"));
		lcdUpdateDisplay();
		printf_P(PSTR("\r\nPlease insert U disk!"));
		while(CH375_IniDisk());
		printf_P(PSTR("\r\nUdisk Initialize OK!"));
		FAT_ReadSector = CH375_ReadOneSector;
		FAT_WriteSector =  CH375_WriteOneSector;
		FAT_ReadCapacity = CH375_ReadDiskSize;
	}
	else
	{
BootLoader:
		printf_P(PSTR("\r\nEnter bootloader for firmware upgrade!"));
		printf_P(PSTR("\r\nPress reset to restart or download the firmware."));
		lcdClrDisBuf();
		LCD_print12_P(0,0,PSTR("Run bootloader  for firmware       upgrade\n    --by bozai"));
		lcdUpdateDisplay();
		cli();
		BootLoaderEntry();
	}

	printf_P(PSTR("\r\nDisk size is: %ld MB"),FAT_ReadCapacity()/1024/1024);
	
	if(FAT_Init())
	{
		printf_P(PSTR("\r\nFAT initialize fialed! Or System error!"));
		printf_P(PSTR("\r\nYou may try to turn off the power and then power on."));
		printf_P(PSTR("\r\nOtherwise pls check that whether your card is correctly fomated"));
		lcdClrDisBuf();
		LCD_print12_P(0,0,PSTR("Wrong FAT!\nSystem halted!\nPls reset the\nsystem! -bozai"));
		lcdUpdateDisplay();
		while(1);
	}//ʼFATļϵͳ
	else 
	{
		printf_P(PSTR("\r\nFAT initialize OK!"));
	}
		
	if(GBK_Ini())
	{
		lcdClrDisBuf();
		LCD_print12_P(0,0,PSTR("Firmware lost!\nPress start to\ncontinue!\n    --By bozai"));
		lcdUpdateDisplay();
		printf_P(PSTR("\r\nFirmware lost!\r\nPress start to continue!"));
		HanziEnable = 0;
	}
	else 
	{
		lcdClrDisBuf();
		LCD_print12_P(0,0,PSTR("̼\nDesign by\nbozai-䲨\nʼ"));
		lcdUpdateDisplay();
		printf_P(PSTR("\r\n\r\n̼"));
		printf_P(PSTR("\r\nDesign by : Michael - 䲨(bozai) 2007-11-01 in ZyXEL"));
		printf_P(PSTR("\r\nEmail: sudazqb@163.com   MSN: zhangqibo_1985@hotmail.com"));
		printf_P(PSTR("\r\n**************** ALL RIGHTS RESERVED! *****************"));
		printf_P(PSTR("\r\n\r\nʼ\r\n"));
              
		HanziEnable = 1;
	}

	while(PINE & MODE);
	
#if FIX_DIRECTORY
	printf_P(PSTR("\r\nSearch the folder %s\r\n"),MUSIC_PATH);
	Search(MUSIC_PATH,&MusicInfo,&totalsongs,&type);//
#else
	printf_P(PSTR("\r\n\r\nSearch & record folder information....."));
	printf_P(PSTR("\r\nNOTE: MAX folder amount is %d for we only use %dBytes EEPROM to store the info\r\n"),(RECORD_ADDR_END-RECORD_ADDR_START)/4 -1,RECORD_ADDR_END-RECORD_ADDR_START);
	SearchInit();
	Search(0,&MusicInfo,&totalsongs,&type);//
#endif

	PlayMusicwithKey();

	while(1);
}

